--------------------------------------------------------------------------------
LIBRARY ieee;
USE ieee.std_logic_1164.ALL;
 
-- Uncomment the following library declaration if using
-- arithmetic functions with Signed or Unsigned values
--USE ieee.numeric_std.ALL;
 
ENTITY fit_testbench IS
END fit_testbench;
 
ARCHITECTURE behavior OF fit_testbench IS 
 
    -- Component Declaration for the Unit Under Test (UUT)
 
    COMPONENT fir
    PORT(
         ap_clk : IN  std_logic;
         ap_rst : IN  std_logic;
         ap_start : IN  std_logic;
         ap_done : OUT  std_logic;
         ap_idle : OUT  std_logic;
         y : OUT  std_logic_vector(15 downto 0);
         y_ap_vld : OUT  std_logic;
         x : IN  std_logic_vector(15 downto 0)
        );
    END COMPONENT;
    

   --Inputs
   signal ap_clk : std_logic := '0';
   signal ap_rst : std_logic := '0';
   signal ap_start : std_logic := '0';
   signal x : std_logic_vector(15 downto 0) := (others => '0');

 	--Outputs
   signal ap_done : std_logic;
   signal ap_idle : std_logic;
   signal y : std_logic_vector(15 downto 0);
   signal y_ap_vld : std_logic;

   -- Clock period definitions
   constant ap_clk_period : time := 10 ns;
	
	constant k : integer := 0; 
	
BEGIN
 
	-- Instantiate the Unit Under Test (UUT)
   uut: fir PORT MAP (
          ap_clk => ap_clk,
          ap_rst => ap_rst,
          ap_start => ap_start,
          ap_done => ap_done,
          ap_idle => ap_idle,
          y => y,
          y_ap_vld => y_ap_vld,
          x => x
        );

   -- Clock process definitions
   ap_clk_process :process
   begin
		ap_clk <= '0';
		wait for ap_clk_period/2;
		ap_clk <= '1';
		wait for ap_clk_period/2;
   end process;
 

   -- Stimulus process
   stim_proc: process
   begin		
      -- hold reset state for 100 ns.
      wait for 100 ns;	
		ap_rst <= '1';
      wait for ap_clk_period*10;
		ap_rst <= '0';
		ap_start <= '1';
		x <= x"8000";
      wait for ap_clk_period;
		ap_start <= '0';
		wait until (ap_done = '1');
      wait for ap_clk_period*3;
		x <= x"0000";
		ap_start <= '1';
      wait for ap_clk_period;
		ap_start <= '0';
		wait until (ap_done = '1');
      wait for ap_clk_period*3;
		ap_start <= '1';
      wait for ap_clk_period;
		ap_start <= '0';
		wait until (ap_done = '1');
      wait for ap_clk_period*3;
		ap_start <= '1';
      wait for ap_clk_period;
		ap_start <= '0';
		wait until (ap_done = '1');
      wait for ap_clk_period*3;
		ap_start <= '1';
      wait for ap_clk_period;
		ap_start <= '0';
		wait until (ap_done = '1');
      wait for ap_clk_period*3;
		ap_start <= '1';
      wait for ap_clk_period;
		ap_start <= '0';
		wait until (ap_done = '1');
      wait for ap_clk_period*3;
		ap_start <= '1';
      wait for ap_clk_period;
		ap_start <= '0';
		wait until (ap_done = '1');
      wait for ap_clk_period*3;
		ap_start <= '1';
      wait for ap_clk_period;
		ap_start <= '0';

      wait;
   end process;

END;
